@@ -1,4 +1,5 @@
 import datetime
+import json
 import logging
 import os
 import subprocess
@@ -13,10 +14,19 @@ import matplotlib.pyplot as plt
 
 from gamutrf.sigwindows import freq_excluded
 from gamutrf.sigwindows import parse_freq_excluded
-from gamutrf.utils import ETTUS_ANT, ETTUS_ARGS, MPL_BACKEND, WIDTH, HEIGHT, DPI
-
+from gamutrf.utils import (
+    ETTUS_ANT,
+    ETTUS_ARGS,
+    MPL_BACKEND,
+    WIDTH,
+    HEIGHT,
+    DPI,
+    DS_PIXELS,
+)
 
+IMSHOW_INTERPOLATION = os.getenv("IMSHOW_INTERPOLATION", "bilinear")
 NFFT = int(os.getenv("NFFT", "0"))
+NFFT_OVERLAP = 512
 SAMPLE_TYPE = "s16"
 MIN_SAMPLE_RATE = int(1e6)
 MAX_SAMPLE_RATE = int(30 * 1e6)
@@ -80,31 +90,65 @@ class SDRRecorder:
     def fft_spectrogram(
         sample_file, fft_file, sample_count, sample_rate, center_freq, nfft
     ):
-        if os.path.exists(fft_file):
-            matplotlib.use(MPL_BACKEND)
-            png_file = sample_file.replace(".zst", ".png")
-            logging.info("generating spectrogram: %s", png_file)
-            i = np.memmap(fft_file, dtype=np.float32, mode="r+")
-            i = np.roll(i.reshape(-1, nfft).swapaxes(0, 1), int(nfft / 2), 0)
-            fig = plt.figure()
-            fig.set_size_inches(WIDTH, HEIGHT)
-            axes = fig.add_subplot(111)
-            axes.set_xlabel("time (s)")
-            axes.set_ylabel("freq (MHz)")
-            fc = center_freq / 1e6
-            fo = sample_rate / 1e6 / 2
-            extent = (0, sample_count / sample_rate, fc - fo, fc + fo)
-            im = axes.imshow(i, cmap="turbo", origin="lower", extent=extent)
-            axes.axis("auto")
-            axes.minorticks_on()
-            plt.colorbar(im, ax=axes)
-            plt.sci(im)
-            plt.savefig(png_file, dpi=DPI)
+        def matplotlib_gc(fig, axes, im):
             axes.images.remove(im)
             fig.clear()
             plt.close()
             plt.cla()
             plt.clf()
+
+        def get_fig(width, height):
+            fig = plt.figure()
+            fig.set_size_inches(width, height)
+            axes = fig.add_subplot(111)
+            return (fig, axes)
+
+        def imshow(axes, i, extent):
+            return axes.imshow(
+                i,
+                cmap="turbo",
+                origin="lower",
+                extent=extent,
+                aspect="auto",
+                interpolation=IMSHOW_INTERPOLATION,
+            )
+
+        def plot_fft(i, sample_file, extent):
+            png_file = sample_file.replace(".zst", ".png")
+            logging.info("generating spectrogram: %s", png_file)
+            fig, axes = get_fig(WIDTH, HEIGHT)
+            axes.set_xlabel("time (s)")
+            axes.set_ylabel("freq (MHz)")
+            axes.axis("auto")
+            axes.minorticks_on()
+            im = imshow(axes, i, extent)
+            plt.colorbar(im, ax=axes)
+            plt.sci(im)
+            plt.savefig(png_file, dpi=DPI)
+            matplotlib_gc(fig, axes, im)
+
+        def plot_ds_fft(i, sample_file, extent):
+            ds_png_file = sample_file.replace(".zst", ".ds.png")
+            logging.info("generating downsampled spectrogram: %s", ds_png_file)
+            fig, axes = get_fig(DS_PIXELS / DPI, DS_PIXELS / DPI)
+            axes.set_axis_off()
+            axes.get_xaxis().set_visible(False)
+            axes.get_yaxis().set_visible(False)
+            im = imshow(axes, i, extent)
+            plt.sci(im)
+            plt.subplots_adjust(left=0, right=1, bottom=0, top=1)
+            plt.savefig(ds_png_file, dpi=DPI)
+            matplotlib_gc(fig, axes, im)
+
+        if os.path.exists(fft_file):
+            matplotlib.use(MPL_BACKEND)
+            i = np.memmap(fft_file, dtype=np.float32, mode="r+")
+            i = np.roll(i.reshape(-1, nfft).swapaxes(0, 1), int(nfft / 2), 0)
+            fc = center_freq / 1e6
+            fo = sample_rate / 1e6 / 2
+            extent = (0, sample_count / sample_rate, fc - fo, fc + fo)
+            plot_fft(i, sample_file, extent)
+            plot_ds_fft(i, sample_file, extent)
             os.remove(fft_file)
 
     def run_recording(
@@ -159,11 +203,12 @@ class SDRRecorder:
 
 
 class EttusRecorder(SDRRecorder):
-
-    # def __init__(self):
-    #    super().__init__()
-    #    # TODO: troubleshoot why this doesn't find an Ettus initially, still.
-    #    # subprocess.call(['uhd_find_devices'])
+    def __init__(self):
+        super().__init__()
+        self.worker_subprocess = None
+        self.last_worker_line = None
+        # TODO: troubleshoot why this doesn't find an Ettus initially, still.
+        # subprocess.call(['uhd_find_devices'])
 
     def record_args(
         self, sample_file, sample_rate, sample_count, center_freq, gain, _agc, rxb
@@ -171,18 +216,14 @@ class EttusRecorder(SDRRecorder):
         # Ettus "nsamps" API has an internal limit, so translate "stream for druation".
         duration = round(sample_count / sample_rate)
         rxb = min(rxb, sample_rate)
+
         args = [
             "/usr/local/bin/mt_rx_samples_to_file",
-            "--file",
-            sample_file,
+            "--json",
             "--rate",
             str(sample_rate),
             "--bw",
             str(sample_rate),
-            "--duration",
-            str(duration),
-            "--freq",
-            str(center_freq),
             "--gain",
             str(gain),
             "--args",
@@ -192,31 +233,65 @@ class EttusRecorder(SDRRecorder):
             "--spb",
             str(rxb),
         ]
+
+        json_args = {
+            "file": sample_file,
+            "duration": duration,
+            "freq": center_freq,
+        }
         if NFFT:
-            args.extend(
-                [
-                    "--nfft",
-                    str(NFFT),
-                    "--nfft_overlap",
-                    str(int(NFFT / 8)),
-                    "--nfft_ds",
-                    str(int(1)),
-                    "--fft_file",
-                    FFT_FILE,
-                ]
+            json_args.update(
+                {
+                    "nfft": NFFT,
+                    "nfft_overlap": NFFT_OVERLAP,
+                    "fft_file": FFT_FILE,
+                }
             )
-        return args
+        return (args, json_args)
 
     def write_recording(
         self, sample_file, sample_rate, sample_count, center_freq, gain, agc, rxb
     ):
         # Ettus doesn't need a wrapper, it can do its own zst compression.
         record_status = -1
-        args = self.record_args(
+        args, json_args = self.record_args(
             sample_file, sample_rate, sample_count, center_freq, gain, agc, rxb
         )
-        logging.info("starting recording: %s", args)
-        record_status = subprocess.check_call(args)
+
+        try:
+
+            def worker_read():
+                self.last_worker_line = (
+                    self.worker_subprocess.stdout.readline().decode("utf-8").strip()
+                )
+                logging.info(self.last_worker_line)
+
+            def worker_write(s):
+                self.worker_subprocess.stdin.write(("%s\n" % s).encode("utf-8"))
+                self.worker_subprocess.stdin.flush()
+
+            if self.worker_subprocess is None:
+                logging.info("starting worker subprocess: %s", args)
+                self.worker_subprocess = subprocess.Popen(
+                    args, stdin=subprocess.PIPE, stdout=subprocess.PIPE
+                )
+                worker_read()
+
+            logging.info("starting recording: %s", json_args)
+            worker_write(json.dumps(json_args))
+            worker_read()
+            last_error = json.loads(self.last_worker_line).get("last_error")
+            if not last_error:
+                record_status = 0
+        except (
+            subprocess.SubprocessError,
+            BrokenPipeError,
+            json.decoder.JSONDecodeError,
+        ) as e:
+            logging.error(e)
+            if self.worker_subprocess:
+                self.worker_subprocess.kill()
+                self.worker_subprocess = None
         return record_status
 
 
