@@ -4,6 +4,7 @@ import json
 import logging
 import os
 import subprocess
+import sys
 import threading
 import time
 
@@ -264,9 +265,11 @@ def call_record_signals(args, lastbins_history, prom_vars):
             signals.extend(list(bins))
         recorder_freq_exclusions = get_freq_exclusions(args)
         recorder_count = len(recorder_freq_exclusions)
-        record_signals = choose_record_signal(signals, recorder_count)
+        record_signals = choose_record_signal(
+            signals, recorder_count * args.max_recorder_signals
+        )
         for signal, recorder in choose_recorders(
-            record_signals, recorder_freq_exclusions
+            record_signals, recorder_freq_exclusions, args.max_recorder_signals
         ):
             signal_hz = int(signal * 1e6)
             record_bps = int(args.record_bw_msps * MB)
@@ -282,7 +285,9 @@ def zstd_file(uncompressed_file):
     subprocess.check_call(["/usr/bin/zstd", "--rm", uncompressed_file])
 
 
-def process_fft_lines(args, prom_vars, buff_file, executor, runonce=False):
+def process_fft_lines(
+    args, prom_vars, buff_file, executor, proxy_result, runonce=False
+):
     lastfreq = 0
     fftbuffer = []
     lastbins_history = []
@@ -304,6 +309,11 @@ def process_fft_lines(args, prom_vars, buff_file, executor, runonce=False):
         openlogts = int(time.time())
         with open(args.log, mode=mode) as l:
             while True:
+                if not proxy_result.running():
+                    logging.error(
+                        "FFT proxy stopped running: %s", proxy_result.result()
+                    )
+                    sys.exit(-1)
                 now = int(time.time())
                 if now - last_fft_report > FFT_BUFFER_TIME * 2:
                     logging.info(
@@ -323,7 +333,8 @@ def process_fft_lines(args, prom_vars, buff_file, executor, runonce=False):
                     os.remove(buff_file)
                 else:
                     schedule.run_pending()
-                    time.sleep(1)
+                    sleep_time = 1
+                    time.sleep(sleep_time)
                     continue
                 lines = txt_buf.splitlines()
                 if not len(lines) > 1:
@@ -351,7 +362,8 @@ def process_fft_lines(args, prom_vars, buff_file, executor, runonce=False):
                 df["scan_pos"] = (df.freq - args.freq_start) / (
                     args.freq_end - args.freq_start
                 )
-                lastfreq = df.freq.iat[-1]
+                if df.size:
+                    lastfreq = df.freq.iat[-1]
                 rotatelognow = False
                 for row in df.itertuples():
                     rollover = row.scan_pos < 0.1 and max_scan_pos > 0.9 and fftbuffer
@@ -419,8 +431,8 @@ def fft_proxy(args, buff_file, buffer_time=FFT_BUFFER_TIME, shutdown_str=None):
 
 def find_signals(args, prom_vars):
     with concurrent.futures.ProcessPoolExecutor(2) as executor:
-        executor.submit(fft_proxy, args, BUFF_FILE)
-        process_fft_lines(args, prom_vars, BUFF_FILE, executor)
+        proxy_result = executor.submit(fft_proxy, args, BUFF_FILE)
+        process_fft_lines(args, prom_vars, BUFF_FILE, executor, proxy_result)
 
 
 def argument_parser():
@@ -537,6 +549,13 @@ def argument_parser():
         default=8001,
         help="Log FFT results from this port",
     )
+    parser.add_argument(
+        "--max_recorder_signals",
+        dest="max_recorder_signals",
+        type=int,
+        default=1,
+        help="Max number of recordings per worker to request",
+    )
     return parser
 
 
